package ssh

import (
	"time"

	cnsms "git.loafle.net/commons/service_matcher-go/ssh"
	ocmsc "git.loafle.net/overflow/commons-go/model/sensorconfig"
	"git.loafle.net/overflow/container_network/crawler/health"
	"git.loafle.net/overflow/crawler-go"
)

type SSHHealthCrawler struct {
	health.SocketHealthCrawler
}

func (c *SSHHealthCrawler) Key() string {
	return "SSH_HEALTH"
}

func (c *SSHHealthCrawler) String() string {
	return "SSH Health Crawler"
}

func (c *SSHHealthCrawler) Auth(auth map[string]interface{}) error {
	return nil
}

func (c *SSHHealthCrawler) Get(config *ocmsc.SensorConfig) (map[string]string, error) {
	result := make(map[string]string, 0)

	conn, err := c.GetConnection(config)
	if err != nil {
		return nil, err
	}
	defer conn.Close()

	for _, mci := range config.MetaCollectionItems {
		switch mci.Key {
		case "ssh.response_time":
			start := time.Now().UTC()
			if err := c.CheckHealth(config, conn); err != nil {
				return nil, err
			}
			elapsed := time.Since(start)
			result[mci.Key] = elapsed.String()
			break
		default:
			break
		}
	}

	return result, nil
}

func NewCrawler() crawler.Crawler {
	c := &SSHHealthCrawler{}
	c.SetMatcher(cnsms.NewMatcher())
	return c
}
